#include <io.h>

#include "common.h"
#include "sensors.h"


// Comment the following line out if leds and beeper should be disabled (e.g. for solarcells)
//#define NOLEDSANDBEEP

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ Common routines

// Beeper
void beepOn(void){ 
#ifndef NOLEDSANDBEEP
  P2OUT |= 0x08;
#endif  
}
void beepOff(void)  { P2OUT &= 0xF7;  }

// red led
void redOn(void)    { 
#ifndef NOLEDSANDBEEP
  P2OUT &= 0xFE;
#endif  
}
void redOff(void)   { P2OUT |= 0x01;  }
void redToggle(void){ P2OUT ^= 0x01;  }

// green led
void greenOn(void)  { 
#ifndef NOLEDSANDBEEP
  P2OUT &= 0xFD;  
#endif
}

void greenOff(void) { P2OUT |= 0x02;  }
void greenToggle(void) { P2OUT ^= 0x02;  }

// yellow led
void yellowOn(void) { 
#ifndef NOLEDSANDBEEP
  P2OUT &= 0xFB;
#endif
}

void yellowOff(void){ P2OUT |= 0x04;  }
void yellowToggle(void){ P2OUT ^= 0x04;  }

// rough estimate: w=1 => 18us, w=10 => 62us, w=100 => 500us
// this method has two big disadvantages: it blocks and it's not accurate, timers are better
void wait(unsigned long int w){ 
  volatile unsigned long int i; 
  for(i=0;i<w;i++) nop(); 
}

// beeps for some time using wait()
void beep(unsigned int j){
#ifndef NOLEDSANDBEEP
	P2OUT |= 0x08;
	wait(j);
	P2OUT &= 0xF7;
#endif
}

void alarmbeep(unsigned long int space, unsigned int times){
  unsigned int i;
  wait(space);
  for(i=0; i<times; i++){
    beep(space);
    wait(space);
  }
}

// generates a random number in 0..n-1 by adding all sensor data together
unsigned char random(unsigned char n){
  long int i = sensorData.lightCount + sensorData.pirCount + sensorData.vibCount + sensorData.micCount;
  i += sensorData.micValue + sensorData.externValue + sensorData.batteryValue + sensorData.temperature + sensorData.fiveMsTimer;
  
  return (i % n);  
}




////////////////////// Float conversion copied from "Embedded Webserver" Project
// unknown helper function
float dimension(int dim){
  float dimension=1.0f;
  while (dim-->1){dimension*=10.0f;}
  while (dim++<0){dimension/=10.0f;}  
  return dimension;
}

// unknown helper function
int getDimension(float f, int dim){
  f/=dimension(dim);
  if (f>5){ // f > 5 
    if (f>8){
      if (f>9){ return 9; }
      else { return 8; }
    } else {
      if (f>7){ return 7; }
      else {
        if (f>6) { return 6; }
	else { return 5;}
      }
    }
  } else { //f<5
    if (f>3){
      if (f>4){ return 4; }
      else { return 3; }
    } else {
      if (f>2){ return 2; }
      else {
        if (f>1) { return 1; }
	else { return 0;}
      }
    }
  }
}

// converts a float into a string, result is in floatbuf
char floatbuf[11];
void setFloat(float f){
  int   dim    = 5;
  int   neg    = 0;
  int   i,zero = 0;
  int   lead   = 1;
 
  if (f<0){
    neg=1;
    floatbuf[0]='-';
    f*=-1;
  }
  if (f>99999.999)f=99999.999;
  
  while (dim>=-3){
    i=getDimension(f,dim);
    f-=i*dimension(dim);
    if (dim>0){
    // we calculate 8 numbers and skip zero leading 0's and have 1 sign possibly
      floatbuf[8-(dim+3)+neg-zero]='0'+i;
    } else {
      // one more because of dot char
      floatbuf[9-(dim+3)+neg-zero]='0'+i;
    }
    // leading zero logic
    if (lead){
      if (i==0 && dim>1){zero++;}
      else {lead=0;}
    }
    // decrease dimension
    dim--;
  }
  // set dot and \0 
  floatbuf[neg+5-zero]='.';
  floatbuf[10-zero]='\0';
  
  //setTextPos(buf,x,y);
}

//////////////////////// Int conversion copied from "Embedded Webserver" Project
// converts an int into a string, result is in in intbuf
char intbuf[8];
int setUnsignedInt(unsigned int hex){
  char tbuf[8];
  int i;
  int value;
  int counter = 0;

  if(hex<=0){ tbuf[0] = '0'; counter=1; }
  else {
    while(hex >= 1){
    
   	value = hex % 10;
    	hex  /= 10;
   	tbuf[counter] =  '0' + value;
  	counter++;
    }  	  
  }
  for(i=0; i<counter; i++){
    intbuf[i] = tbuf[counter-1-i];
  }
  intbuf[counter] = '\0';
  
  return counter;  
}


// converts a long int into a string, result is in longintbuf
char longintbuf[10];
int setUnsignedLongInt(unsigned long int hex){
  char tbuf[10];
  int i;
  unsigned long int value;
  int counter = 0;

  if(hex<=0){ tbuf[0] = '0'; counter=1; }
  else {
    while(hex >= 1){
    
   	value = hex % 10;
    	hex  /= 10;
   	tbuf[counter] =  '0' + value;
  	counter++;
    }  	  
  }
  for(i=0; i<counter; i++){
    longintbuf[i] = tbuf[counter-1-i];
  }
  longintbuf[counter] = '\0';
  
  return counter;  
}



// converts an int into a string, result is written to *c
int writeUnsignedInt(unsigned int hex, char* c){
  char tbuf[8];
  int i;
  int value;
  int counter = 0;

  if(hex<=0){ tbuf[0] = '0'; counter=1; }
  else {
    while(hex >= 1){
    
   	value = hex % 10;
    	hex  /= 10;
   	tbuf[counter] =  '0' + value;
  	counter++;
    }  	  
  }
  for(i=0; i<counter; i++){
    c[i] = tbuf[counter-1-i];
  }
  c[counter] = '\0';
  
  return counter;  
}



// second int conversion routine for a fixed number of digits
// same as before, but maximum number of digits/bytes written is num		
unsigned int writeUnsignedIntFixed(unsigned int value, char* c, unsigned int num){
  int i = num;
  unsigned int t = value;
  
  while(i>0){
    i--;
    c[i] = (t % 10) + 0x30;
    t /= 10;    
  }
  return num;
}

unsigned int writeUnsignedLongIntFixed(unsigned long int value, char* c, unsigned int num){
  int i = num;
  unsigned long int t = value;
  
  while(i>0){
    i--;
    c[i] = (t % 10) + 0x30;
    t /= 10;    
  }
  return num;
}




// method to read an int from a string
unsigned int readUnsignedInt(char* c, unsigned int num){
  unsigned int i = 0;
  unsigned int value = 0;
  
  while(num > 0){
    value = value * 10;
    value = value + (c[i] & 0x0F);
    num--;
    i++;    
  }
  return value;
}

unsigned long int readUnsignedLongInt(char* c, unsigned int num){
  unsigned int i = 0;
  unsigned long int value = 0;
  
  while(num > 0){
    value = value * 10;
    value = value + (c[i] & 0x0F);
    num--;
    i++;    
  }
  return value;
}	    
	    

// Hex Conversions
// converts the byte *b into the string cs[0] cs[1]
void fromByteToHex(unsigned char* b, unsigned char* cs){
  switch((*b & 0xF0)>>4){
    case 0x0:   cs[0] = '0'; break; 
    case 0x1:   cs[0] = '1'; break; 
    case 0x2:   cs[0] = '2'; break; 
    case 0x3:   cs[0] = '3'; break; 
    case 0x4:   cs[0] = '4'; break; 
    case 0x5:   cs[0] = '5'; break; 
    case 0x6:   cs[0] = '6'; break; 
    case 0x7:   cs[0] = '7'; break; 
    case 0x8:   cs[0] = '8'; break; 
    case 0x9:   cs[0] = '9'; break; 
    case 0xA:   cs[0] = 'A'; break; 
    case 0xB:   cs[0] = 'B'; break; 
    case 0xC:   cs[0] = 'C'; break; 
    case 0xD:   cs[0] = 'D'; break; 
    case 0xE:   cs[0] = 'E'; break; 
    case 0xF:   cs[0] = 'F'; break; 
  }
      
  switch(*b & 0x0F){
    case 0x0:   cs[1] = '0'; break; 
    case 0x1:   cs[1] = '1'; break; 
    case 0x2:   cs[1] = '2'; break; 
    case 0x3:   cs[1] = '3'; break; 
    case 0x4:   cs[1] = '4'; break; 
    case 0x5:   cs[1] = '5'; break; 
    case 0x6:   cs[1] = '6'; break; 
    case 0x7:   cs[1] = '7'; break; 
    case 0x8:   cs[1] = '8'; break; 
    case 0x9:   cs[1] = '9'; break; 
    case 0xA:   cs[1] = 'A'; break; 
    case 0xB:   cs[1] = 'B'; break; 
    case 0xC:   cs[1] = 'C'; break; 
    case 0xD:   cs[1] = 'D'; break; 
    case 0xE:   cs[1] = 'E'; break; 
    case 0xF:   cs[1] = 'F'; break; 
  }    
}

// converts the string at cs[0] cs[1] into a byte at *b
void fromHexToByte(unsigned char* cs, unsigned char* b){
  *b = 0;
  
  if((cs[0] >= '0') && (cs[0] <= '9')){
      *b = (cs[0] - '0') * 16;
  } else { // expects cs[0] to be A-F
      *b = (cs[0] - 'A' + 10) * 16;      
  }
  
  if((cs[1] >= '0') && (cs[1] <= '9')){
      *b += (cs[1] - '0');
  } else { // expects cs[1] to be A-F
      *b += (cs[1] - 'A' + 10);      
  }    
}

