#ifndef _EEPROMH_
#define _EEPROMH_

#include "sensors.h"

#define CONFIG_ADDRESS    (0x0000)      // the address where the config starts
#define EVENTMEM_ADDRESS  (0x0030)      // the address where the eventmemory starts
#define MAX_ADDRESS       (0x1FFF)      // maximum address of the eeprom (8K)

#define EVENT_SIZE        (sizeof(sensorData))                            // size of an event
#define EVENTMEM_SIZE     (MAX_ADDRESS - EVENTMEM_ADDRESS) / EVENT_SIZE   // number of possible evententries


int writeEEPROMByte(char high_adr, char low_adr, char data); //write one single byte into the eeprom
char readEEPROMByte(char high_adr, char low_adr);            //read one single byte from the eeprom
void writeEEPROM(int e_adr, char* r_adr, int size);          //write several bytes into the eeprom
void readEEPROM(int e_adr, char* r_adr, int size);           //read several bytes from the eeprom

// Configuration of the MSP device is written at CONFIG_ADDRESS (0x0000) in the EEProm
// the configuration is automatically loaded into the eepromConfig-struct (see sensors.h)
// at boot time
// if any method changes a value of eepromConfig it should call saveConfig afterwards
// if saveConfig isnt called after changes, they will be lost after the next reset!

struct config {
  unsigned int id;                  // Id of this MSP device
  unsigned char announceFlags;      // if events occur this variable says
                                    // where they get announced ( -> EEPROM, -> Serial, -> Funk)

  unsigned int micInterval;         // Interval around the micAverage
  unsigned int batteryLimit;        // if batteryValue falls under batteryLimit events will be registered
  unsigned int externLimit;         // same as batteryValue for the extern voltage
  //unsigned int rxActivityLimit;     // first threshold to notice someone is sending
  unsigned int rxReceiveLimit;      // second threshold, represents CarrierDetect
  unsigned long int lowerLightLimit; // events will occur if lightCounter not between lower and 
  unsigned long int upperLightLimit; // upper limit
  
  unsigned char transmitPower;      // power for transmittting
  char smsnr[12];                   // mobile's number for receiving sms
} eepromConfig;

void loadConfig(void);
void saveConfig(void);


void eepromInit(void); // finds the current eventMemPointer for the eventlog, called at boottime
void eeprom_appendEvent(struct sensorDataRecord* event); // appends an event into the mem
extern unsigned int eventMemPointer; // position where next event will get written

#endif
