#include <signal.h>
#include <io.h>

#include "common.h"
#include "serial.h"
#include "sensors.h"
#include "recir.h"
#include "clock.h"
#include "terminal.h"
#include "watchdog.h"
#include "eeprom.h"
#include "funk.h"
#include "ehandler.h"
#include "sendir.h"
#include "sms.h"

void initPorts(void){
  ////////// Port 1 ////
  P1SEL = 0x00;
  P1DIR = 0x81;       // Outputs: P10=IRSend, P17=RS232RTS
                      // Inputs: P11=Light, P12=IRRec, P13=PIR, P14=Vibration, 
                      //         P15=Clockalarm, P16=RS232CTS
  P1OUT = 0x00;

  ////////// Port 2 ////
  P2SEL = 0x00;       // No Sels
  P2DIR = 0x7F;       // Outpus: P20..P23=Leds+Beeper, P24..P26=Poti
                      // Inputs: P27=Taster
  P2OUT = 0x77;
  
  ////////// Port 3 ////
  P3SEL = 0xE0;       // Sels for P34..P37 to activate UART, 
  P3DIR = 0x5F;       // Inputs: P30..P33=CON4, P35/P37=RXD Transceiver/RS232
                      // OutPuts: P36/P38=TXD Transceiver/RS232
  P3OUT = 0xE0;       // Output a Zero on P34(TXD Transceiver) and turn SELECT off when receiving!!!

  ////////// Port 4 ////
  P4SEL = 0x00;       // CON5 Stecker
  P4DIR = 0xFF;
  P4OUT = 0x00;
  
  ////////// Port 5 ////
  P5SEL = 0x00;       // P50/P51= Clock SDA/SCL, P52/P53/P54=EEPROM SDA/SCL/WP
  P5DIR = 0xDA;       // P56/P57=Transceiver CNTRL0/1
  P5OUT = 0x0F;
  
  ////////// Port 6 ////
  P6SEL = 0x00;       // P60=Microphone, P61=PIR digital (same as P13), P62=PIR analog
  P6DIR = 0x00;       // P63=extern voltage, P64=battery voltage, P65=Receive power
  P6OUT = 0x00;
}

#define BOOTBEEP    1000


int main(void)
{    
  // bootsequence ;)
  initPorts();        redOn();      beep(BOOTBEEP); 
  initUart();         yellowOn();   beep(BOOTBEEP);
  loadConfig();       greenOn();    beep(BOOTBEEP);
  recir_init();       redOff();     beep(BOOTBEEP);
  initClock();        yellowOff();  beep(BOOTBEEP);
  eepromInit();       greenOff();   beep(BOOTBEEP);

  greenOn(); redOn();
  initFunk();
  eint();     

  greenOff(); redOff(); yellowOn();


  sensor_init(); 
  yellowOff();
  beep(BOOTBEEP*15);
  
  if(!(eepromConfig.announceFlags & AF_HANDY)){
    sendRS232String("\n================= SENSOR BOOTED ====================\n");

  printConfig();
  sendRS232String("====================================================\n\n");
  }

  // do mobile initializing if needed
  if(eepromConfig.announceFlags & AF_HANDY){
    if(!initMobile()) eepromConfig.announceFlags &= ~AF_HANDY; // fall back into normal mode if mobile's not present
    saveConfig();
  }
 
  // OS 8)
  for(;;){
    startWatchdog(); // starts&resets the watchdog, long procedures should call
                     // stop watchdog, else MSP will reset
    
    terminal_main();      // handler for the terminal program
    checkLocalEvents();  // handler for local events
    funkPacketHandler();  // handler for incoming radio packets

    
    // UNCOMMENT THIS FOR TESTING:
    // turns green on when lightpulswidth is measured (>800Hz)
    // turns yellow on when lightpulscount is measured (<490Hz)    
    //if(P1SEL & 0x02){ greenOn(); yellowOff(); }
    //else            { greenOff(); yellowOn(); }
    
    // UNCOMMENT THIS FOR TESTING:
    // turn red on if carrier is detected
    // if(funkCD()) redOn(); else redOff(); 
    
    
    // UNCOMMENT THIS FOR TESTING
    // switches green/yellow led when CTS is detected
    //if(testCTS()){ greenOn(); yellowOff(); }
    //else         { greenOff(); yellowOn(); }         
  }  
}

