#include <io.h>
#include "sms.h"
#include "serial.h"
#include "sensors.h"
#include "ehandler.h"
#include "common.h"
#include "clock.h"
#include "terminal.h"

struct timeRecord lastSMS; // when was the last sms sent?
int smsSent = 0;           // how many sms were sent since last reboot?

// sends one ATcommand to the mobile and waits for the OK
// if no OK is returned within some time the msp beeps 2 times, if
// the mobile replies ERROR it beeps 4 times...
unsigned char sendATCommand(char* command){
  unsigned long int waitc;
  serReceivedFlags &= ~(SER_OK_RECEIVED + SER_ERROR_RECEIVED); // clear flags
  wait(10000);
  
  sendRS232String(command);                                    // send command
  sendRS232('\r');
  for(waitc=0; waitc<SMS_WAITFOROK; waitc++){                  // wait some time for the OK
    if(serReceivedFlags & SER_OK_RECEIVED){ return 1; }        // OK received, return 1
    else if(serReceivedFlags & SER_ERROR_RECEIVED){            // ERROR received, beep and return 0
      alarmbeep(5000, 4);
      return 0; 
    }    
  }
  alarmbeep(5000, 2);                                          // nothing received, beep and return 0
  return 0;          
}

// prepares handy for receiving the SMS body
// if this method returns 0, no sms can be sent because too many sms already sent
// or last sms was to short before
unsigned char prepareSendSMS(char* nr){
  struct timeRecord cTime = sensorData.timeStamp;
  long int diff = timeStampDiff(&lastSMS, &cTime);
  
  if(diff > SMS_PAUSE && ((SMS_MAX==0) || (smsSent < SMS_MAX))){ // check sms limits
    lastSMS = sensorData.timeStamp;
    smsSent++;
    
    serReceivedFlags &= ~(SER_PROMPT_RECEIVED); // clear flags, send AT+CMGS="..." and wait for the prompt
    sendRS232String("AT+CMGS=\""); sendRS232Chars(nr, 11); sendRS232String("\"\r");
    while(!(serReceivedFlags & SER_PROMPT_RECEIVED));

    wait(100000);

    return 1;
  } else {
    /*sendRS232String("SMS nicht gesendet, diff="); sendRS232Int(diff); sendRS232String(" smsSent="); sendRS232Int(smsSent); sendRS232('\n');*/
    return 0;
  }
}

// finishes the SMS after prepareSendSMS
void finishSMS(void){ sendRS232(CTRLZ); sendRS232(CTRLZ); sendRS232(CTRLZ); }

// sends some Text as SMS
void sendSMS(char* nr, char* txt){
  if(prepareSendSMS(nr)){
    sendRS232String(txt);
    finishSMS();
  }
}

// sends an Event as SMS
void sendSMSEvent(char* nr, struct sensorDataRecord* event){
  if(prepareSendSMS(nr)){
    printEventToSerial(event, 0);
    finishSMS();
  }    
}

// inits the mobile
unsigned char initMobile(void){
  wait(10000);
  sendRS232String("\r");
  wait(10000);
  if(!sendATCommand("ATE0")) return 0;              // echo off
  if(!sendATCommand("AT+CMGF=1")) return 0;         // textmode on (for sms)
  if(!sendATCommand("AT+CNMI=2,2,0,0,0")) return 0; // tells the handy to pass sms to the serial port

  return 1;
}
